@@ -1152,7 +1152,7 @@ has_all_certs(void)
 }
 
 void
-dcc::init(void)
+dcc_init(void)
 {
 	if (config_bool("dcc", true) && !g_icb_mode) {
 		struct integer_context intctx("dcc_port", 1024, 65535, 8080);
@@ -1171,7 +1171,7 @@ dcc::init(void)
 }
 
 void
-dcc::deinit(void)
+dcc_deinit(void)
 {
 	tls_server::end();
 
