@@ -114,9 +114,12 @@ static struct tagConfDefValues {
 	{ "qbot_host",                 TYPE_STRING,  1, "CServe.quakenet.org" },
 
 	/* IRCv3 */
-	{ "account_notify",            TYPE_BOOLEAN, 2, "no" },
+	{ "account_notify",            TYPE_BOOLEAN, 2, "yes" },
+	{ "account_tag",               TYPE_BOOLEAN, 2, "yes" },
 	{ "away_notify",               TYPE_BOOLEAN, 2, "no" },
 	{ "batch",                     TYPE_BOOLEAN, 3, "yes" },
+	{ "chghost",                   TYPE_BOOLEAN, 3, "yes" },
+	{ "extended_join",             TYPE_BOOLEAN, 2, "yes" },
 	{ "invite_notify",             TYPE_BOOLEAN, 2, "no" },
 	{ "multi_prefix",              TYPE_BOOLEAN, 2, "yes" },
 	{ "server_time",               TYPE_BOOLEAN, 2, "yes" },
