@@ -59,6 +59,7 @@ struct quit_context {
 	CSTRING		nick, user, host;
 };
 
+static void	auto_op(CSTRING, CSTRING) NONNULL;
 static void	handle_quit(PIRC_WINDOW, PPRINTTEXT_CONTEXT,
 		    struct quit_context *) NONNULL;
 static bool	is_netsplit(CSTRING, std::string &, std::string &) NONNULL;
@@ -106,19 +107,76 @@ event_chan_hp(struct irc_message_compo *compo)
 	}
 }
 
+static void
+auto_op(CSTRING channel, CSTRING nick)
+{
+	if (g_am_irc_op &&
+	    config_bool("auto_op_yourself", true) &&
+	    (net_send("MODE %s +o %s", channel, nick) < 0 ||
+	    net_send("SAMODE %s +o %s", channel, nick) < 0))
+		throw std::runtime_error("cannot send");
+}
+
+static void
+join_perform_some_tasks(CSTRING channel, CSTRING nick, CSTRING account,
+    CSTRING rl_name)
+{
+	PNAMES n = NULL;
+
+	if (account != NULL && *account == ':')
+		account++;
+	if (rl_name != NULL && *rl_name == ':')
+		rl_name++;
+
+	if (!is_irc_channel(channel) ||
+	    strpbrk(channel + 1, g_forbidden_chan_name_chars) != NULL) {
+		throw std::runtime_error("bogus irc channel");
+	} else if (strings_match_ignore_case(nick, g_my_nickname)) {
+		if (spawn_chat_window(channel, "No title.") != 0)
+			throw std::runtime_error("cannot spawn chat window");
+		auto_op(channel, nick);
+	} else if (event_names_htbl_insert(nick, channel) != OK) {
+		throw std::runtime_error("unable to add user to channel list");
+	} else if (account != NULL && rl_name != NULL &&
+	    (n = event_names_htbl_lookup(nick, channel)) != NULL) {
+		free(n->account);
+		free(n->rl_name);
+		n->account = sw_strdup(account);
+		n->rl_name = sw_strdup(rl_name);
+	}
+}
+
+static void
+chk_split(CSTRING nick, CSTRING channel, netsplit *&split)
+{
+	if (!netsplit_db_empty() &&
+	    (split = netsplit_find(nick, channel)) != NULL) {
+		if (split->remove_nick(nick)) {
+			if (!split->join_begun())
+				split->set_join_time(time(NULL));
+		} else {
+			throw std::runtime_error("failed to remove nick from "
+			    "split");
+		}
+	}
+}
+
 /* event_join
 
-   Example:
-     :<nick>!<user>@<host> JOIN <channel> */
+   Examples:
+     :<nick>!<user>@<host> JOIN <channel>
+     :<nick>!<user>@<host> JOIN <channel> <account> :<real name>
+     :<nick>!<user>@<host> JOIN <channel> * :<real name> */
 void
 event_join(struct irc_message_compo *compo)
 {
 	PRINTTEXT_CONTEXT	ctx;
 
 	try {
+		CSTRING		 channel, account, rl_name;
 		CSTRING		 nick, user, host;
 		STRING		 prefix = NULL;
-		STRING		 state = const_cast<STRING>("");
+		STRING		 state[2];
 		netsplit	*split = NULL;
 
 		if (compo == NULL)
@@ -126,47 +184,32 @@ event_join(struct irc_message_compo *compo)
 		else if (compo->prefix == NULL)
 			throw std::runtime_error("no prefix");
 
+		channel = account = rl_name = NULL;
 		prefix = &compo->prefix[1];
+		state[0] = state[1] = const_cast<STRING>("");
 
-		if ((nick = strtok_r(prefix, "!@", &state)) == NULL)
+		if ((nick = strtok_r(prefix, "!@", &state[0])) == NULL)
 			throw std::runtime_error("no nickname");
-		if ((user = strtok_r(NULL, "!@", &state)) == NULL)
+		if ((user = strtok_r(NULL, "!@", &state[0])) == NULL)
 			user = "<no user>";
-		if ((host = strtok_r(NULL, "!@", &state)) == NULL)
+		if ((host = strtok_r(NULL, "!@", &state[0])) == NULL)
 			host = "<no host>";
 
-		immutable_cp_t channel = (*(compo->params) == ':' ?
-		    &compo->params[1] : &compo->params[0]);
+		const int num = strFeed(compo->params, 2);
 
-		if (!is_irc_channel(channel) ||
-		    strpbrk(channel + 1, g_forbidden_chan_name_chars) != NULL) {
-			throw std::runtime_error("bogus irc channel");
-		} else if (strings_match_ignore_case(nick, g_my_nickname)) {
-			if (spawn_chat_window(channel, "No title.") != 0) {
-				throw std::runtime_error("cannot spawn "
-				    "chat window");
-			}
+		channel = strtok_r(compo->params, "\n", &state[1]);
+		account = strtok_r(NULL, "\n", &state[1]);
+		rl_name = strtok_r(NULL, "\n", &state[1]);
 
-			if (g_am_irc_op &&
-			    config_bool("auto_op_yourself", true) &&
-			    (net_send("MODE %s +o %s", channel, nick) < 0 ||
-			    net_send("SAMODE %s +o %s", channel, nick) < 0))
-				throw std::runtime_error("cannot send");
-		} else if (event_names_htbl_insert(nick, channel) != OK) {
-			throw std::runtime_error("unable to add user to "
-			    "channel list");
-		}
+		if (!num && (account != NULL || rl_name != NULL))
+			throw std::runtime_error("too many tokens");
+		else if (channel == NULL)
+			throw std::runtime_error("no channel");
+		else if (*channel == ':')
+			channel++;
 
-		if (!netsplit_db_empty() &&
-		    (split = netsplit_find(nick, channel)) != NULL) {
-			if (split->remove_nick(nick)) {
-				if (!split->join_begun())
-					split->set_join_time(time(NULL));
-			} else {
-				throw std::runtime_error("failed to remove "
-				    "nick from split");
-			}
-		}
+		join_perform_some_tasks(channel, nick, account, rl_name);
+		chk_split(nick, channel, split);
 
 		if (split == NULL && config_bool("joins_parts_quits", true)) {
 			printtext_context_init(&ctx, NULL, TYPE_SPEC1_SPEC2,
@@ -554,12 +597,16 @@ static int
 RemoveAndInsertNick(const char *old_nick, const char *new_nick,
     const char *label)
 {
-	PNAMES	p;
-	bool	is_owner, is_superop, is_op, is_halfop, is_voice;
+	PNAMES		p;
+	bool		is_owner, is_superop, is_op, is_halfop, is_voice;
+	std::string	array[2];
 
 	if ((p = event_names_htbl_lookup(old_nick, label)) == NULL)
 		return ERR; /* non-fatal: old_nick not found on channel */
 
+	array[0].assign(p->account ? p->account : "<no account>");
+	array[1].assign(p->rl_name ? p->rl_name : "<no rl name>");
+
 	is_owner	= p->is_owner;
 	is_superop	= p->is_superop;
 	is_op		= p->is_op;
@@ -572,6 +619,15 @@ RemoveAndInsertNick(const char *old_nick, const char *new_nick,
 	} else if (event_names_htbl_insert(new_nick, label) != OK) {
 		err_log(0, "%s: event_names_htbl_insert", __func__);
 		return ERR;
+	} else if ((p = event_names_htbl_lookup(new_nick, label)) != NULL) {
+		free(p->account);
+		free(p->rl_name);
+		p->account = sw_strdup(array[0].c_str());
+		p->rl_name = sw_strdup(array[1].c_str());
+	} else {
+		err_log(0, "%s: new nick (%s) not found on %s", __func__,
+		    new_nick, label);
+		return ERR;
 	}
 
 	/*
@@ -826,9 +882,9 @@ handle_quit(PIRC_WINDOW window, PPRINTTEXT_CONTEXT ptext_ctx,
 				    __func__);
 			}
 		} else {
-			const std::string str(ctx->nick);
-
 			if (!split->join_begun()) {
+				const std::string str(ctx->nick);
+
 #if defined(__cplusplus) && __cplusplus >= 201103L
 				split->nicks.emplace_back(str);
 #else
