@@ -37,6 +37,7 @@
 #include "irc.h"
 #include "libUtils.h"
 #include "main.h"
+#include "messagetags.h"
 #include "nestHome.h"
 #include "network.h"
 #include "printtext.h"
@@ -52,6 +53,7 @@
 #include "events/batch.h"
 #include "events/cap.h"
 #include "events/channel.h"
+#include "events/chghost.h"
 #include "events/error.h"
 #include "events/invite.h"
 #include "events/list.h"
@@ -98,14 +100,15 @@ static HANDLE		nickname_mtx;
 #endif
 
 static struct normal_events_tag {
-	char			*normal_event;
-	event_handler_fn	 event_handler;
+	CSTRING			normal_event;
+	event_handler_fn	event_handler;
 } normal_events[] = {
 	{ "ACCOUNT",      event_account      },
 	{ "AUTHENTICATE", event_authenticate },
 	{ "AWAY",         event_away         },
 	{ "BATCH",        event_batch        },
 	{ "CAP",          event_cap          },
+	{ "CHGHOST",      event_chghost      },
 	{ "ERROR",        event_error        },
 	{ "INVITE",       event_invite       },
 	{ "JOIN",         event_join         },
@@ -124,11 +127,11 @@ static struct normal_events_tag {
 };
 
 static struct numeric_events_tag {
-	char			*numeric_event;
-	char			*official_name;
-	enum to_window		 window;
-	int			 ext_bits;
-	event_handler_fn	 event_handler;
+	CSTRING			numeric_event;
+	CSTRING			official_name;
+	enum to_window		window;
+	int			ext_bits;
+	event_handler_fn	event_handler;
 } numeric_events[] = {
 	{ "001", "RPL_WELCOME",             NO_WINDOW,      0, event_welcome },
 	{ "002", "RPL_YOURHOST",            STATUS_WINDOW,  1, NULL },
@@ -421,170 +424,50 @@ irc_extract_msg(struct irc_message_compo *compo, PIRC_WINDOW to_window,
 	}
 }
 
-static int
-get_num_semicolons(const char *str)
-{
-	int num = 0;
-
-	for (const char *cp = str; *cp != '\0'; cp++) {
-		if (*cp == ';')
-			num++;
-	}
-
-	return num;
-}
-
-static int
-handle_batch(const size_t bytes, char *substring, const char *protocol_message)
-{
-	char		 ref[250] = { '\0' };
-	const char	*pmsg_plus_off;
-	size_t		 offset = bytes;
-
-	pmsg_plus_off = &protocol_message[offset];
-
-	if (strings_match(pmsg_plus_off, "")) {
-		printf_and_free(substring, _("%s: empty message"), __func__);
-		return -1;
-	} else {
-		debug("%s: protocol_message + offset: \"%s\"", __func__,
-		    pmsg_plus_off);
-	}
-
-	debug("%s: substring = \"%s\"", __func__, substring);
-	debug("%s: offset = '%zu'", __func__, offset);
-
-	if (strchr(substring, ';') == NULL) {
-		if (sw_strcpy(ref, &substring[7], sizeof ref) != 0) {
-			printf_and_free(substring, "%s: error assigning batch "
-			    "ref tag", __func__);
-			return -1;
-		} else if (strings_match(ref, "")) {
-			printf_and_free(substring, "%s: empty ref tag",
-			    __func__);
-			return -1;
-		}
-
-		event_batch_add_irc_msgs(ref, pmsg_plus_off);
-	} else { /* has semicolons */
-		char	*cp, *str;
-		int	 year, month, day;
-		int	 hour, minute, second, precision;
-
-		year = month = day = 0;
-		hour = minute = second = precision = 0;
-
-		if (get_num_semicolons(substring) > 1) {
-			printf_and_free(substring, _("%s: too many semicolons"),
-			    __func__);
-			return -1;
-		}
-
-		cp = substring;
-		cp += strlen("@batch=");
-		offset = strcspn(cp, ";");
-		cp[offset] = '\0';
-
-		debug("%s: cp = \"%s\"", __func__, cp);
-		debug("%s: offset = '%zu'", __func__, offset);
-
-		if (sw_strcpy(ref, cp, sizeof ref) != 0) {
-			printf_and_free(substring, "%s: too long ref tag",
-			    __func__);
-			return -1;
-		} else if (strings_match(ref, "")) {
-			printf_and_free(substring, "%s: empty ref tag",
-			    __func__);
-			return -1;
-		}
-
-		cp[offset] = ';';
-
-		if ((cp = strstr(substring, ";time=")) == NULL) {
-			printf_and_free(substring, _("%s: cannot find time"),
-			    __func__);
-			return -1;
-		} else if (sscanf(cp, ";time=%d-%d-%dT%d:%d:%d.%dZ",
-		    &year, &month, &day,
-		    &hour, &minute, &second, &precision) != 7) {
-			printf_and_free(substring, _("%s: server time error"),
-			    __func__);
-			return -1;
-		}
-
-		str = strdup_printf("@time=%d-%d-%dT%d:%d:%d.%dZ %s",
-		    year, month, day,
-		    hour, minute, second, precision,
-		    pmsg_plus_off);
-		event_batch_add_irc_msgs(ref, str);
-		free(str);
-	}
-
-	free(substring);
-	return -1;
-}
-
 static int
 handle_extension(size_t *bytes, const char *protocol_message,
     struct irc_message_compo *compo)
 {
-	char *substring;
+	char			*substring;
+	struct messagetags	*tags;
+	static const size_t	 min_bytes = 1;
+	static const size_t	 max_bytes = 900;
+
+	if ((*bytes = strcspn(protocol_message, " ")) < min_bytes ||
+	    *bytes > max_bytes) {
+		err_log(ERANGE, "%s: too few/many message tags", __func__);
+		return -1;
+	}
 
-	*bytes = strcspn(protocol_message, " ");
 	substring = xmalloc((*bytes) + 1);
 	memcpy(substring, protocol_message, *bytes);
 	substring[*bytes] = '\0';
+	tags = msgtags_get(substring);
+	free_and_null(&substring);
 
-/*
- * sscanf() is safe in this context
- */
-#if WIN32
-#pragma warning(disable: 4996)
-#endif
-	if (!strncmp(substring, "@batch=", 7)) {
-		return handle_batch(*bytes, substring, protocol_message);
-	} else if (!strncmp(substring, "@time=", 6)) {
-		if (sscanf(substring, "@time=%d-%d-%dT%d:%d:%d.%dZ",
-		    & (compo->year),
-		    & (compo->month),
-		    & (compo->day),
-		    & (compo->hour),
-		    & (compo->minute),
-		    & (compo->second),
-		    & (compo->precision)) != 7) {
-			printf_and_free(substring, _("%s: server time error"),
-			    __func__);
-			return -1;
-		}
-	} else {
-		printf_and_free(substring, _("%s: unsupported extension"),
+	if (isNull(tags->account) &&
+	    isNull(tags->batch) &&
+	    isNull(tags->srv_time)) {
+		printtext_print("err", _("%s: unsupported extension"),
 		    __func__);
+		msgtags_free(tags);
 		return -1;
+	} else if (tags->batch != NULL) {
+		msgtags_handle_batch(addrof(protocol_message[*bytes]), tags);
+		msgtags_free(tags);
+		return -1;
+	} else {
+		msgtags_process(compo, tags);
+		msgtags_free(tags);
 	}
-/*
- * Reset warning behavior to its default value
- */
-#if WIN32
-#pragma warning(default: 4996)
-#endif
 
-	free(substring);
 	return 0;
 }
 
-/**
- * Sort message components - into prefix, command and params.
- */
-static struct irc_message_compo *
-SortMsgCompo(const char *protocol_message)
+static void
+compo_init(struct irc_message_compo *compo)
 {
-	char				*cp = NULL, *state = "";
-	char				*remaining_data = NULL;
-	int				 message_has_prefix = 0,
-					 requested_feeds = -1;
-	short int			 loop_run = 0;
-	size_t				 bytes = 0;
-	struct irc_message_compo	*compo = xcalloc(sizeof *compo, 1);
+	compo->account = NULL;
 
 	compo->year = compo->month = compo->day = -1;
 	compo->hour = compo->minute = compo->second = compo->precision = -1;
@@ -592,36 +475,20 @@ SortMsgCompo(const char *protocol_message)
 	compo->prefix = NULL;
 	compo->command = NULL;
 	compo->params = NULL;
+}
 
-	if (*protocol_message == '@') {
-		if (handle_extension(&bytes, protocol_message, compo) == -1) {
-			free(compo);
-			return NULL;
-		}
-	} /* ===== EOF IRCv3 extensions ===== */
-
-	const char *ccp = &protocol_message[bytes];
-	while (*ccp == ' ')
-		ccp++;
-	message_has_prefix = (*ccp == ':');
-	requested_feeds = (message_has_prefix ? 2 : 1);
-	remaining_data = sw_strdup(ccp);
-
-	if (strFeed(remaining_data, requested_feeds) != requested_feeds &&
-	    strstr(remaining_data, "\nAWAY") == NULL) {
-		free(compo);
-		printf_and_free(remaining_data, "In %s: strFeed: "
-		    "requested feeds mismatch feeds written", __func__);
-		return NULL;
-	}
-
-	loop_run = 0;
-	cp = &remaining_data[0];
+static void
+compo_tokenize(STRING str, const int message_has_prefix,
+	       struct irc_message_compo *compo)
+{
+	STRING		state = "";
+	short int	loop_run = 0;
 
 	while (true) {
-		const char *token;
+		CSTRING token;
 
-		if ((token = strtok_r(cp, "\n", &state)) == NULL)
+		if ((token = strtok_r(loop_run == 0 ? str : NULL, "\n",
+		    &state)) == NULL)
 			break;
 
 		switch (loop_run) {
@@ -650,10 +517,48 @@ SortMsgCompo(const char *protocol_message)
 			break;
 
 		loop_run++;
-		cp = NULL;
+	}
+}
+
+/**
+ * Sort message components - into prefix, command and params.
+ */
+static struct irc_message_compo *
+SortMsgCompo(const char *protocol_message)
+{
+	char				*remaining_data = NULL;
+	int				 message_has_prefix = 0,
+					 requested_feeds = -1;
+	size_t				 bytes = 0;
+	struct irc_message_compo	*compo = xcalloc(sizeof *compo, 1);
+
+	compo_init(compo);
+
+	if (*protocol_message == '@') {
+		if (handle_extension(&bytes, protocol_message, compo) == -1) {
+			free(compo);
+			return NULL;
+		}
+	} /* ===== EOF IRCv3 extensions ===== */
+
+	const char *cp = &protocol_message[bytes];
+	while (*cp == ' ')
+		cp++;
+	message_has_prefix = (*cp == ':');
+	requested_feeds = (message_has_prefix ? 2 : 1);
+	remaining_data = sw_strdup(cp);
+
+	if (strFeed(remaining_data, requested_feeds) != requested_feeds &&
+	    strstr(remaining_data, "\nAWAY") == NULL) {
+		free(compo);
+		printf_and_free(remaining_data, "In %s: strFeed: "
+		    "requested feeds mismatch feeds written", __func__);
+		return NULL;
 	}
 
+	compo_tokenize(&remaining_data[0], message_has_prefix, compo);
 	free(remaining_data);
+
 	if (compo->command == NULL || (compo->params == NULL &&
 	    !strings_match(compo->command, "AWAY"))) {
 		FreeMsgCompo(compo);
@@ -668,6 +573,8 @@ SortMsgCompo(const char *protocol_message)
 static void
 FreeMsgCompo(struct irc_message_compo *compo)
 {
+	free(compo->account);
+
 	free(compo->prefix);
 	free(compo->command);
 	free(compo->params);
@@ -794,6 +701,12 @@ get_last_token(const char *buffer)
 	return sw_strdup(++ last_token);
 }
 
+static inline bool
+is_terminated_recvchunk(const char last)
+{
+	return (last == '\r' || last == '\n');
+}
+
 /**
  * Handle and interpret irc events
  */
@@ -801,7 +714,6 @@ void
 irc_handle_interpret_events(char *recvbuffer, char **message_concat,
     enum message_concat_state *state)
 {
-	bool			 terminated_recvchunk = false;
 	char			*cp = NULL, *tokstate = "";
 	char			*last_token = NULL;
 	long int		 loop_count = 0;
@@ -820,17 +732,7 @@ irc_handle_interpret_events(char *recvbuffer, char **message_concat,
 		*state = CONCAT_BUFFER_IS_EMPTY;
 	}
 
-	switch (recvbuffer[strlen(recvbuffer) - 1]) {
-	case '\r':
-	case '\n':
-		terminated_recvchunk = true;
-		break;
-	default:
-		terminated_recvchunk = false;
-		break;
-	}
-
-	if (!terminated_recvchunk)
+	if (!is_terminated_recvchunk(recvbuffer[strlen(recvbuffer) - 1]))
 		last_token = get_last_token(recvbuffer); /* Must be freed */
 
 	cp = &recvbuffer[0];
