@@ -87,7 +87,7 @@
 /* Things with external linkage
    ============================ */
 
-chararray_t	g_swircVersion	= "v3.5.0";
+chararray_t	g_swircVersion	= "v3.5.1";
 chararray_t	g_swircYear	= "2012-2024";
 chararray_t	g_swircAuthor	= "Markus Uhlin";
 chararray_t	g_swircWebAddr	= "https://www.nifty-networks.net/swirc/";
@@ -113,6 +113,7 @@ bool	g_debug_logging		= false;
 bool	g_explicit_config_file	= false;
 bool	g_force_tls		= false;
 bool	g_icb_mode		= false;
+bool	g_ircv3_extensions	= true;
 bool	g_sasl_authentication	= true;
 bool	g_ssl_verify_peer	= true;
 
@@ -150,6 +151,7 @@ static stringarray_t OptionDesc = {
   "                         launched by a toast.\n",
 #endif
   N_("    -W <password>        Equal effect as flag 'p' but non-interactive\n"),
+  N_("    -X                   Disable all IRCv3 extensions\n"),
   N_("    -c <server[:port]>   Connect to IRC server\n"),
   N_("    -d                   Debug logging\n"),
   N_("    -i                   Turn on Internet Citizen's Band mode\n"),
@@ -433,6 +435,9 @@ process_options(int argc, char *argv[], const char *optstring)
 		case 'W':
 			case_password(false);
 			break;
+		case 'X':
+			g_ircv3_extensions = false;
+			break;
 		case 'c':
 			case_connect();
 			break;
@@ -654,7 +659,7 @@ main(int argc, char *argv[])
 	}
 #endif
 
-	process_options(argc, argv, "46CPRSTW:c:dh:ij:n:pr:u:x:");
+	process_options(argc, argv, "46CPRSTW:Xc:dh:ij:n:pr:u:x:");
 
 	srand(get_seed());
 
@@ -760,7 +765,11 @@ main(int argc, char *argv[])
 	/*
 	 * Reverse order...
 	 */
+	dcc_deinit();
 	net_ssl_deinit();
+#ifdef WIN32
+	winsock_deinit();
+#endif
 	readline_deinit();
 	windowSystem_deinit();
 	statusbar_deinit();
