@@ -523,6 +523,47 @@ windowSystem_deinit(void)
 #endif
 }
 
+static void
+set_nick_as_title(STRING title, CSTRING nick, size_t size)
+{
+	if (sw_strcpy(title, nick, size) != 0)
+		BZERO(title, size);
+}
+
+CSTRING
+make_window_title(CSTRING nick)
+{
+	static char title[400] = { '\0' };
+
+	if (!atomic_load_bool(&g_on_air) ||
+	    !config_bool("extended_join", true) ||
+	    !g_ircv3_extensions ||
+	    g_icb_mode) {
+		set_nick_as_title(title, nick, sizeof title);
+		return addrof(title[0]);
+	}
+
+	FOREACH_HASH_TABLE_ENTRY() {
+		FOREACH_WINDOW_IN_ENTRY() {
+			PNAMES n;
+
+			if (!is_irc_channel(window->label))
+				continue;
+			n = event_names_htbl_lookup(nick, window->label);
+			if (n == NULL ||
+			    n->account == NULL ||
+			    n->rl_name == NULL)
+				continue;
+			(void) snprintf(title, sizeof title, "%s (%s): %s",
+			    nick, n->account, n->rl_name);
+			return addrof(title[0]);
+		}
+	}
+
+	set_nick_as_title(title, nick, sizeof title);
+	return addrof(title[0]);
+}
+
 /**
  * Return the window identified by the given @label -- or NULL on
  * error
