@@ -1,5 +1,5 @@
-#ifndef WINDOW_H
-#define WINDOW_H
+#ifndef SRC_WINDOW_H_
+#define SRC_WINDOW_H_
 
 #if defined(PANEL_HDR)
 #include PANEL_HDR
@@ -24,6 +24,9 @@ typedef enum {
 
 typedef struct tagNAMES {
 	char	*nick;
+	char	*account;
+	char	*rl_name;
+
 	bool	 is_owner;
 	bool	 is_superop;
 	bool	 is_op;
@@ -81,6 +84,7 @@ void windowSystem_deinit(void);
 /*lint -sem(get_list_of_matching_channels, r_null) */
 /*lint -sem(get_list_of_matching_queries, r_null) */
 
+CSTRING		make_window_title(CSTRING);
 PIRC_WINDOW	window_by_label(CSTRING);
 PIRC_WINDOW	window_by_refnum(int);
 PTEXTBUF	get_list_of_matching_channels(CSTRING search_var);
